"""
tamqp.

An adaptable Python library to interact with your choice of cloud RabbitMQ providrs. Originally designed and developed at Taekus.
"""

__version__ = "0.0.1"
__author__ = 'Saurabh Gupta'
__credits__ = 'Taekus Inc.'